@@ -165,12 +165,21 @@ pub mod error;
 pub mod accelerator;
 pub mod event;
 pub mod event_loop;
+pub mod global_shortcut;
 mod icon;
 pub mod keyboard;
 pub mod menu;
 pub mod monitor;
 mod platform_impl;
-#[cfg(any(target_os = "windows", target_os = "linux", target_os = "macos"))]
+#[cfg(any(
+  target_os = "windows",
+  target_os = "macos",
+  target_os = "linux",
+  target_os = "dragonfly",
+  target_os = "freebsd",
+  target_os = "netbsd",
+  target_os = "openbsd"
+))]
 #[cfg(feature = "tray")]
 pub mod system_tray;
 pub mod window;
