@@ -10,10 +10,13 @@ use crate::{
   event_loop::{EventLoop, EventLoopWindowTarget},
   menu::CustomMenuItem,
   monitor::MonitorHandle,
-  platform_impl::get_aux_state_mut,
+  platform_impl::{get_aux_state_mut, Parent},
   window::{Window, WindowBuilder},
 };
 
+#[cfg(feature = "tray")]
+use crate::system_tray::{SystemTray, SystemTrayBuilder};
+
 use cocoa::{appkit, base::id};
 
 /// Additional methods on `Window` that are specific to MacOS.
@@ -301,6 +304,8 @@ impl NativeImage {
 ///  - `with_titlebar_buttons_hidden`
 ///  - `with_fullsize_content_view`
 pub trait WindowBuilderExtMacOS {
+  /// Sets a parent to the window to be created.
+  fn with_parent_window(self, parent: *mut c_void) -> WindowBuilder;
   /// Enables click-and-drag behavior for the entire window, not just the titlebar.
   fn with_movable_by_window_background(self, movable_by_window_background: bool) -> WindowBuilder;
   /// Makes the titlebar transparent and allows the content to appear behind it.
@@ -320,6 +325,12 @@ pub trait WindowBuilderExtMacOS {
 }
 
 impl WindowBuilderExtMacOS for WindowBuilder {
+  #[inline]
+  fn with_parent_window(mut self, parent: *mut c_void) -> WindowBuilder {
+    self.platform_specific.parent = Parent::ChildOf(parent);
+    self
+  }
+
   #[inline]
   fn with_movable_by_window_background(
     mut self,
@@ -451,3 +462,37 @@ impl<T> EventLoopWindowTargetExtMacOS for EventLoopWindowTarget<T> {
     unsafe { msg_send![app, hideOtherApplications: 0] }
   }
 }
+
+#[cfg(feature = "tray")]
+pub trait SystemTrayBuilderExtMacOS {
+  /// Sets the icon as a [template](https://developer.apple.com/documentation/appkit/nsimage/1520017-template?language=objc).
+  ///
+  /// Images you mark as template images should consist of only black and clear colors.
+  /// You can use the alpha channel in the image to adjust the opacity of black content.
+  ///
+  fn with_icon_as_template(self, is_template: bool) -> Self;
+}
+
+#[cfg(feature = "tray")]
+impl SystemTrayBuilderExtMacOS for SystemTrayBuilder {
+  fn with_icon_as_template(mut self, is_template: bool) -> Self {
+    self.0.system_tray.icon_is_template = is_template;
+    self
+  }
+}
+
+#[cfg(feature = "tray")]
+pub trait SystemTrayExtMacOS {
+  /// Sets the icon as a [template](https://developer.apple.com/documentation/appkit/nsimage/1520017-template?language=objc).
+  ///
+  /// You need to update this value before changing the icon.
+  ///
+  fn set_icon_as_template(&mut self, is_template: bool);
+}
+
+#[cfg(feature = "tray")]
+impl SystemTrayExtMacOS for SystemTray {
+  fn set_icon_as_template(&mut self, is_template: bool) {
+    self.0.icon_is_template = is_template
+  }
+}
