@@ -630,7 +630,10 @@ impl Window {
 
   pub fn show_menu(&self) {}
 
-  pub fn set_skip_taskbar(&self, _skip: bool) {}
+  pub fn is_menu_visible(&self) -> bool {
+    warn!("`Window::is_menu_visible` is ignored on Android");
+    false
+  }
 
   pub fn set_cursor_icon(&self, _: window::CursorIcon) {}
 
