@@ -8,18 +8,17 @@ use std::{
   sync::atomic::{AtomicBool, AtomicI32, Ordering},
 };
 
-use gdk::{Cursor, EventKey, EventMask, WindowEdge, WindowExt, WindowState};
+use gdk::{WindowEdge, WindowState};
 use gdk_pixbuf::{Colorspace, Pixbuf};
 use gtk::{prelude::*, AccelGroup, ApplicationWindow, Orientation};
 
 use crate::{
   dpi::{PhysicalPosition, PhysicalSize, Position, Size},
   error::{ExternalError, NotSupportedError, OsError as RootOsError},
-  event::ElementState,
   icon::{BadIcon, Icon},
   menu::{MenuId, MenuItem},
   monitor::MonitorHandle as RootMonitorHandle,
-  window::{CursorIcon, Fullscreen, UserAttentionType, WindowAttributes},
+  window::{CursorIcon, Fullscreen, UserAttentionType, WindowAttributes, BORDERLESS_RESIZE_INSET},
 };
 
 use super::{
@@ -104,7 +103,7 @@ impl Window {
     let app = &event_loop_window_target.app;
     let window_requests_tx = event_loop_window_target.window_requests_tx.clone();
     let window = gtk::ApplicationWindow::new(app);
-    let window_id = WindowId(window.get_id());
+    let window_id = WindowId(window.id());
     event_loop_window_target
       .windows
       .borrow_mut()
@@ -114,7 +113,7 @@ impl Window {
     window.add_accel_group(&accel_group);
 
     // Set Width/Height & Resizable
-    let win_scale_factor = window.get_scale_factor();
+    let win_scale_factor = window.scale_factor();
     let (width, height) = attributes
       .inner_size
       .map(|size| size.to_logical::<f64>(win_scale_factor as f64).into())
@@ -170,8 +169,8 @@ impl Window {
 
     // Set Transparent
     if attributes.transparent {
-      if let Some(screen) = window.get_screen() {
-        if let Some(visual) = screen.get_rgba_visual() {
+      if let Some(screen) = window.screen() {
+        if let Some(visual) = screen.rgba_visual() {
           window.set_visual(Some(&visual));
         }
       }
@@ -179,7 +178,7 @@ impl Window {
       window.connect_draw(|_, cr| {
         cr.set_source_rgba(0., 0., 0., 0.);
         cr.set_operator(cairo::Operator::Source);
-        cr.paint();
+        let _ = cr.paint();
         cr.set_operator(cairo::Operator::Over);
         Inhibit(false)
       });
@@ -209,33 +208,6 @@ impl Window {
     window.set_visible(attributes.visible);
     window.set_decorated(attributes.decorations);
 
-    if !attributes.decorations && attributes.resizable {
-      window.add_events(EventMask::POINTER_MOTION_MASK | EventMask::BUTTON_MOTION_MASK);
-
-      window.connect_motion_notify_event(|window, event| {
-        if let Some(gdk_window) = window.get_window() {
-          let (cx, cy) = event.get_root();
-          hit_test(&gdk_window, cx, cy);
-        }
-        Inhibit(false)
-      });
-
-      window.connect_button_press_event(|window, event| {
-        if event.get_button() == 1 {
-          if let Some(gdk_window) = window.get_window() {
-            let (cx, cy) = event.get_root();
-            let result = hit_test(&gdk_window, cx, cy);
-
-            // this check is necessary, otherwise the window won't recieve the click properly when resize isn't needed
-            if result != WindowEdge::__Unknown(8) {
-              window.begin_resize_drag(result, 1, cx as i32, cy as i32, event.get_time());
-            }
-          }
-        }
-        Inhibit(false)
-      });
-    }
-
     window.set_keep_above(attributes.always_on_top);
     if let Some(icon) = attributes.window_icon {
       window.set_icon(Some(&icon.inner.into()));
@@ -247,34 +219,34 @@ impl Window {
       window.hide();
     }
 
-    let w_pos = window.get_position();
+    let w_pos = window.position();
     let position: Rc<(AtomicI32, AtomicI32)> = Rc::new((w_pos.0.into(), w_pos.1.into()));
     let position_clone = position.clone();
 
-    let w_size = window.get_size();
+    let w_size = window.size();
     let size: Rc<(AtomicI32, AtomicI32)> = Rc::new((w_size.0.into(), w_size.1.into()));
     let size_clone = size.clone();
 
     window.connect_configure_event(move |_, event| {
-      let (x, y) = event.get_position();
+      let (x, y) = event.position();
       position_clone.0.store(x, Ordering::Release);
       position_clone.1.store(y, Ordering::Release);
 
-      let (w, h) = event.get_size();
+      let (w, h) = event.size();
       size_clone.0.store(w as i32, Ordering::Release);
       size_clone.1.store(h as i32, Ordering::Release);
 
       false
     });
 
-    let w_max = window.get_property_is_maximized();
+    let w_max = window.is_maximized();
     let maximized: Rc<AtomicBool> = Rc::new(w_max.into());
     let max_clone = maximized.clone();
     let minimized = Rc::new(AtomicBool::new(false));
     let min_clone = minimized.clone();
 
     window.connect_window_state_event(move |_window, event| {
-      let state = event.get_new_window_state();
+      let state = event.new_window_state();
       max_clone.store(state.contains(WindowState::MAXIMIZED), Ordering::Release);
       min_clone.store(state.contains(WindowState::ICONIFIED), Ordering::Release);
       Inhibit(false)
@@ -282,38 +254,14 @@ impl Window {
 
     let scale_factor: Rc<AtomicI32> = Rc::new(win_scale_factor.into());
     let scale_factor_clone = scale_factor.clone();
-    window.connect_property_scale_factor_notify(move |window| {
-      scale_factor_clone.store(window.get_scale_factor(), Ordering::Release);
+    window.connect_scale_factor_notify(move |window| {
+      scale_factor_clone.store(window.scale_factor(), Ordering::Release);
     });
 
     if let Err(e) = window_requests_tx.send((window_id, WindowRequest::WireUpEvents)) {
       log::warn!("Fail to send wire up events request: {}", e);
     }
 
-    let window_requests_tx_clone = window_requests_tx.clone();
-    window.connect_key_press_event(move |_window, event_key| {
-      if let Err(e) = window_requests_tx_clone.send((
-        window_id,
-        WindowRequest::KeyboardInput((event_key.to_owned(), ElementState::Pressed)),
-      )) {
-        log::warn!("Fail to send user attention request: {}", e);
-      }
-
-      Inhibit(false)
-    });
-
-    let window_requests_tx_clone = window_requests_tx.clone();
-    window.connect_key_release_event(move |_window, event_key| {
-      if let Err(e) = window_requests_tx_clone.send((
-        window_id,
-        WindowRequest::KeyboardInput((event_key.to_owned(), ElementState::Released)),
-      )) {
-        log::warn!("Fail to send user attention request: {}", e);
-      }
-
-      Inhibit(false)
-    });
-
     window.queue_draw();
 
     let win = Self {
@@ -506,11 +454,11 @@ impl Window {
   }
 
   pub fn is_resizable(&self) -> bool {
-    self.window.get_resizable()
+    self.window.is_resizable()
   }
 
   pub fn is_decorated(&self) -> bool {
-    self.window.get_decorated()
+    self.window.is_decorated()
   }
 
   #[inline]
@@ -626,18 +574,19 @@ impl Window {
   }
 
   pub fn current_monitor(&self) -> Option<RootMonitorHandle> {
-    let screen = self.window.get_display().get_default_screen();
-    let window = self.window.get_window().unwrap();
-    let number = screen.get_monitor_at_window(&window);
-    let handle = MonitorHandle::new(&self.window.get_display(), number);
+    let screen = self.window.display().default_screen();
+    let window = self.window.window().unwrap();
+    #[allow(deprecated)] // Gtk3 Window only accepts Gdkscreen
+    let number = screen.monitor_at_window(&window);
+    let handle = MonitorHandle::new(&self.window.display(), number);
     Some(RootMonitorHandle { inner: handle })
   }
 
   #[inline]
   pub fn available_monitors(&self) -> VecDeque<MonitorHandle> {
     let mut handles = VecDeque::new();
-    let display = self.window.get_display();
-    let numbers = display.get_n_monitors();
+    let display = self.window.display();
+    let numbers = display.n_monitors();
 
     for i in 0..numbers {
       let monitor = MonitorHandle::new(&display, i);
@@ -648,9 +597,10 @@ impl Window {
   }
 
   pub fn primary_monitor(&self) -> Option<RootMonitorHandle> {
-    let screen = self.window.get_display().get_default_screen();
-    let number = screen.get_primary_monitor();
-    let handle = MonitorHandle::new(&self.window.get_display(), number);
+    let screen = self.window.display().default_screen();
+    #[allow(deprecated)] // Gtk3 Window only accepts Gdkscreen
+    let number = screen.primary_monitor();
+    let handle = MonitorHandle::new(&self.window.display(), number);
     Some(RootMonitorHandle { inner: handle })
   }
 
@@ -697,21 +647,16 @@ pub enum WindowRequest {
   Redraw,
   Menu((Option<MenuItem>, Option<MenuId>)),
   SetMenu((Option<menu::Menu>, AccelGroup, gtk::MenuBar)),
-  KeyboardInput((EventKey, ElementState)),
   GlobalHotKey(u16),
 }
 
 pub fn hit_test(window: &gdk::Window, cx: f64, cy: f64) -> WindowEdge {
-  let (left, top) = window.get_position();
-  let (w, h) = (window.get_width(), window.get_height());
+  let (left, top) = window.position();
+  let (w, h) = (window.width(), window.height());
   let (right, bottom) = (left + w, top + h);
   let (cx, cy) = (cx as i32, cy as i32);
 
-  let fake_border = 3; // change this to manipulate how far inside the window, the resize can happen
-
-  let display = window.get_display();
-
-  const LEFT: i32 = 0b00001;
+  const LEFT: i32 = 0b0001;
   const RIGHT: i32 = 0b0010;
   const TOP: i32 = 0b0100;
   const BOTTOM: i32 = 0b1000;
@@ -720,12 +665,13 @@ pub fn hit_test(window: &gdk::Window, cx: f64, cy: f64) -> WindowEdge {
   const BOTTOMLEFT: i32 = BOTTOM | LEFT;
   const BOTTOMRIGHT: i32 = BOTTOM | RIGHT;
 
-  let result = (LEFT * (if cx < (left + fake_border) { 1 } else { 0 }))
-    | (RIGHT * (if cx >= (right - fake_border) { 1 } else { 0 }))
-    | (TOP * (if cy < (top + fake_border) { 1 } else { 0 }))
-    | (BOTTOM * (if cy >= (bottom - fake_border) { 1 } else { 0 }));
+  #[rustfmt::skip]
+  let result = (LEFT * (if cx < (left + BORDERLESS_RESIZE_INSET) { 1 } else { 0 }))
+    | (RIGHT * (if cx >= (right - BORDERLESS_RESIZE_INSET) { 1 } else { 0 }))
+    | (TOP * (if cy < (top + BORDERLESS_RESIZE_INSET) { 1 } else { 0 }))
+    | (BOTTOM * (if cy >= (bottom - BORDERLESS_RESIZE_INSET) { 1 } else { 0 }));
 
-  let edge = match result {
+  match result {
     LEFT => WindowEdge::West,
     TOP => WindowEdge::North,
     RIGHT => WindowEdge::East,
@@ -734,31 +680,10 @@ pub fn hit_test(window: &gdk::Window, cx: f64, cy: f64) -> WindowEdge {
     TOPRIGHT => WindowEdge::NorthEast,
     BOTTOMLEFT => WindowEdge::SouthWest,
     BOTTOMRIGHT => WindowEdge::SouthEast,
-    // has to be bigger than 7. otherwise it will match the number with a variant of WindowEdge enum and we don't want to do that
-    // also if the number ever change, makke sure to change it in the connect_button_press_event for window and webview
+    // we return `WindowEdge::__Unknown` to be ignored later.
+    // we must return 8 or bigger, otherwise it will be the same as one of the other 7 variants of `WindowEdge` enum.
     _ => WindowEdge::__Unknown(8),
-  };
-
-  // FIXME: calling `window.begin_resize_drag` seems to revert the cursor back to normal style
-  window.set_cursor(
-    Cursor::from_name(
-      &display,
-      match edge {
-        WindowEdge::North => "n-resize",
-        WindowEdge::South => "s-resize",
-        WindowEdge::East => "e-resize",
-        WindowEdge::West => "w-resize",
-        WindowEdge::NorthWest => "nw-resize",
-        WindowEdge::NorthEast => "ne-resize",
-        WindowEdge::SouthEast => "se-resize",
-        WindowEdge::SouthWest => "sw-resize",
-        _ => "default",
-      },
-    )
-    .as_ref(),
-  );
-
-  edge
+  }
 }
 
 impl Drop for Window {
