@@ -41,7 +41,6 @@ use crate::{
   event::{DeviceEvent, Event, Force, RawKeyEvent, Touch, TouchPhase, WindowEvent},
   event_loop::{ControlFlow, EventLoopClosed, EventLoopWindowTarget as RootELW},
   keyboard::{KeyCode, ModifiersState},
-  menu::{MenuId, MenuType},
   monitor::MonitorHandle as RootMonitorHandle,
   platform_impl::platform::{
     accelerator,
@@ -571,13 +570,8 @@ lazy_static! {
         winuser::RegisterWindowMessageA("Tao::SetRetainMaximized\0".as_ptr() as LPCSTR)
     };
     static ref THREAD_EVENT_TARGET_WINDOW_CLASS: Vec<u16> = unsafe {
-        use std::ffi::OsStr;
-        use std::os::windows::ffi::OsStrExt;
 
-        let class_name: Vec<_> = OsStr::new("Tao Thread Event Target")
-            .encode_wide()
-            .chain(Some(0).into_iter())
-            .collect();
+        let class_name= util::to_wstring("Tao Thread Event Target");
 
         let class = winuser::WNDCLASSEXW {
             cbSize: mem::size_of::<winuser::WNDCLASSEXW>() as UINT,
@@ -1020,7 +1014,7 @@ unsafe fn public_window_callback_inner<T: 'static>(
         let new_monitor = winuser::MonitorFromRect(&new_rect, winuser::MONITOR_DEFAULTTONULL);
         match fullscreen {
           Fullscreen::Borderless(ref mut fullscreen_monitor) => {
-            if new_monitor != ptr::null_mut()
+            if !new_monitor.is_null()
               && fullscreen_monitor
                 .as_ref()
                 .map(|monitor| new_monitor != monitor.inner.hmonitor())
@@ -1086,17 +1080,6 @@ unsafe fn public_window_callback_inner<T: 'static>(
       result = ProcResult::Value(0);
     }
 
-    // this is catched by the accelerator translator
-    winuser::WM_COMMAND => {
-      let menu_id = LOWORD(wparam as u32) as u16;
-      subclass_input.send_event(Event::MenuEvent {
-        window_id: Some(RootWindowId(WindowId(window))),
-        menu_id: MenuId(menu_id as u16),
-        // todo fix menutype
-        origin: MenuType::MenuBar,
-      });
-    }
-
     // this is necessary for us to maintain minimize/restore state
     winuser::WM_SYSCOMMAND => {
       if wparam == winuser::SC_RESTORE {
@@ -1903,7 +1886,7 @@ unsafe fn public_window_callback_inner<T: 'static>(
       let win_flags = subclass_input.window_state.lock().window_flags();
 
       if !win_flags.contains(WindowFlags::DECORATIONS) {
-        // adjust the maximized borderless window to fill the work area rectangle of the display monitor
+        // adjust the maximized borderless window so it doesn't cover the taskbar
         if util::is_maximized(window) {
           let monitor = monitor::current_monitor(window);
           if let Ok(monitor_info) = monitor::get_monitor_info(monitor.hmonitor()) {
@@ -1911,7 +1894,7 @@ unsafe fn public_window_callback_inner<T: 'static>(
             params.rgrc[0] = monitor_info.rcWork;
           }
         }
-        result = ProcResult::Value(0);
+        result = ProcResult::Value(0); // return 0 here to make the windowo borderless
       } else {
         result = ProcResult::DefSubclassProc;
       }
@@ -1922,9 +1905,7 @@ unsafe fn public_window_callback_inner<T: 'static>(
         let win_flags = state.window_flags();
 
         // Only apply this hit test for borderless windows that wants to be resizable
-        if !win_flags.contains(WindowFlags::DECORATIONS)
-          && win_flags.contains(WindowFlags::RESIZABLE)
-        {
+        if !win_flags.contains(WindowFlags::DECORATIONS) {
           // cursor location
           let (cx, cy) = (
             windowsx::GET_X_LPARAM(lparam),
