@@ -2,7 +2,7 @@
 // SPDX-License-Identifier: Apache-2.0
 
 use raw_window_handle::RawWindowHandle;
-use std::{collections::HashMap, ffi::CString, fmt, os::windows::ffi::OsStrExt, sync::Mutex};
+use std::{collections::HashMap, ffi::CString, fmt, sync::Mutex};
 
 use winapi::{
   shared::{basetsd, minwindef, windef},
@@ -17,7 +17,7 @@ use crate::{
   window::WindowId as RootWindowId,
 };
 
-use super::{accelerator::register_accel, keyboard::key_to_vk, WindowId};
+use super::{accelerator::register_accel, keyboard::key_to_vk, util::to_wstring, WindowId};
 
 #[derive(Copy, Clone)]
 struct AccelWrapper(winuser::ACCEL);
@@ -30,47 +30,44 @@ impl fmt::Debug for AccelWrapper {
 const CUT_ID: usize = 5001;
 const COPY_ID: usize = 5002;
 const PASTE_ID: usize = 5003;
-const HIDE_ID: usize = 5004;
-const CLOSE_ID: usize = 5005;
-const QUIT_ID: usize = 5006;
-const MINIMIZE_ID: usize = 5007;
+const SELECT_ALL_ID: usize = 5004;
+const HIDE_ID: usize = 5005;
+const CLOSE_ID: usize = 5006;
+const QUIT_ID: usize = 5007;
+const MINIMIZE_ID: usize = 5008;
 
 lazy_static! {
-  static ref MENU_IDS: Mutex<Vec<usize>> = Mutex::new(vec![]);
+  static ref MENU_IDS: Mutex<Vec<u16>> = Mutex::new(vec![]);
 }
 
 pub struct MenuHandler {
   window_id: Option<RootWindowId>,
   menu_type: MenuType,
-  send_event: Box<dyn Fn(Event<'static, ()>)>,
+  event_sender: Box<dyn Fn(Event<'static, ()>)>,
 }
 
 impl MenuHandler {
   pub fn new(
-    send_event: Box<dyn Fn(Event<'static, ()>)>,
+    event_sender: Box<dyn Fn(Event<'static, ()>)>,
     menu_type: MenuType,
     window_id: Option<RootWindowId>,
   ) -> MenuHandler {
     MenuHandler {
       window_id,
-      send_event,
       menu_type,
+      event_sender,
     }
   }
-  pub fn send_click_event(&self, menu_id: u16) {
-    // we send only tray event as the window event
-    // is catched into the main event_loop process.
-    if self.menu_type == MenuType::ContextMenu {
-      (self.send_event)(Event::MenuEvent {
-        menu_id: MenuId(menu_id),
-        origin: self.menu_type,
-        window_id: self.window_id,
-      });
-    }
+  pub fn send_menu_event(&self, menu_id: u16) {
+    (self.event_sender)(Event::MenuEvent {
+      menu_id: MenuId(menu_id),
+      origin: self.menu_type,
+      window_id: self.window_id,
+    });
   }
 
   pub fn send_event(&self, event: Event<'static, ()>) {
-    (self.send_event)(event);
+    (self.event_sender)(event);
   }
 }
 
@@ -125,7 +122,7 @@ impl MenuItemAttributes {
 
 #[derive(Debug, Clone)]
 pub struct Menu {
-  pub(crate) hmenu: windef::HMENU,
+  hmenu: windef::HMENU,
   accels: HashMap<u16, AccelWrapper>,
 }
 
@@ -159,10 +156,8 @@ impl Menu {
     }
   }
 
-  pub fn into_hmenu(self) -> windef::HMENU {
-    let hmenu = self.hmenu;
-    std::mem::forget(self);
-    hmenu
+  pub fn hmenu(&self) -> windef::HMENU {
+    self.hmenu
   }
 
   // Get the accels table
@@ -229,7 +224,7 @@ impl Menu {
       winuser::AppendMenuW(
         self.hmenu,
         flags,
-        submenu.into_hmenu() as _,
+        submenu.hmenu() as _,
         to_wstring(&title).as_mut_ptr(),
       );
     }
@@ -270,6 +265,14 @@ impl Menu {
           to_wstring("&Paste\tCtrl+V").as_mut_ptr(),
         );
       },
+      MenuItem::SelectAll => unsafe {
+        winuser::AppendMenuW(
+          self.hmenu,
+          winuser::MF_STRING,
+          SELECT_ALL_ID,
+          to_wstring("&Select all\tCtrl+A").as_mut_ptr(),
+        );
+      },
       MenuItem::Hide => unsafe {
         winuser::AppendMenuW(
           self.hmenu,
@@ -327,6 +330,8 @@ impl Menu {
   }
 */
 
+const MENU_SUBCLASS_ID: usize = 4568;
+
 pub fn initialize(
   menu_builder: Menu,
   window_handle: RawWindowHandle,
@@ -334,10 +339,15 @@ pub fn initialize(
 ) -> Option<windef::HMENU> {
   if let RawWindowHandle::Windows(handle) = window_handle {
     let sender: *mut MenuHandler = Box::into_raw(Box::new(menu_handler));
-    let menu = menu_builder.clone().into_hmenu();
+    let menu = menu_builder.clone().hmenu();
 
     unsafe {
-      commctrl::SetWindowSubclass(handle.hwnd as _, Some(subclass_proc), 0, sender as _);
+      commctrl::SetWindowSubclass(
+        handle.hwnd as _,
+        Some(subclass_proc),
+        MENU_SUBCLASS_ID,
+        sender as _,
+      );
       winuser::SetMenu(handle.hwnd as _, menu);
     }
 
@@ -351,75 +361,76 @@ pub fn initialize(
   }
 }
 
-pub(crate) fn to_wstring(str: &str) -> Vec<u16> {
-  std::ffi::OsStr::new(str)
-    .encode_wide()
-    .chain(Some(0).into_iter())
-    .collect()
-}
-
 pub(crate) unsafe extern "system" fn subclass_proc(
   hwnd: windef::HWND,
   msg: minwindef::UINT,
   wparam: minwindef::WPARAM,
   lparam: minwindef::LPARAM,
   _id: basetsd::UINT_PTR,
-  data: basetsd::DWORD_PTR,
+  subclass_input_ptr: basetsd::DWORD_PTR,
 ) -> minwindef::LRESULT {
-  let proxy = &mut *(data as *mut MenuHandler);
+  let subclass_input_ptr = subclass_input_ptr as *mut MenuHandler;
+  let subclass_input = &*(subclass_input_ptr);
+
+  if msg == winuser::WM_DESTROY {
+    Box::from_raw(subclass_input_ptr);
+  }
+
   match msg {
     winuser::WM_COMMAND => {
       match wparam {
         CUT_ID => {
-          execute_edit_command(EditCommands::Cut);
+          execute_edit_command(EditCommand::Cut);
         }
         COPY_ID => {
-          execute_edit_command(EditCommands::Copy);
+          execute_edit_command(EditCommand::Copy);
         }
         PASTE_ID => {
-          execute_edit_command(EditCommands::Paste);
+          execute_edit_command(EditCommand::Paste);
+        }
+        SELECT_ALL_ID => {
+          execute_edit_command(EditCommand::SelectAll);
         }
         HIDE_ID => {
           winuser::ShowWindow(hwnd, winuser::SW_HIDE);
         }
         CLOSE_ID => {
-          proxy.send_event(Event::WindowEvent {
+          subclass_input.send_event(Event::WindowEvent {
             window_id: RootWindowId(WindowId(hwnd)),
             event: WindowEvent::CloseRequested,
           });
         }
         QUIT_ID => {
-          proxy.send_event(Event::LoopDestroyed);
+          subclass_input.send_event(Event::LoopDestroyed);
         }
         MINIMIZE_ID => {
           winuser::ShowWindow(hwnd, winuser::SW_MINIMIZE);
         }
         _ => {
-          if MENU_IDS.lock().unwrap().contains(&wparam) {
-            proxy.send_click_event(wparam as u16);
+          let menu_id = minwindef::LOWORD(wparam as _);
+          if MENU_IDS.lock().unwrap().contains(&menu_id) {
+            subclass_input.send_menu_event(menu_id);
           }
         }
       }
       0
     }
-    winuser::WM_DESTROY => {
-      Box::from_raw(data as *mut MenuHandler);
-      0
-    }
     _ => commctrl::DefSubclassProc(hwnd, msg, wparam, lparam),
   }
 }
 
-enum EditCommands {
+enum EditCommand {
   Copy,
   Cut,
   Paste,
+  SelectAll,
 }
-fn execute_edit_command(command: EditCommands) {
+fn execute_edit_command(command: EditCommand) {
   let key = match command {
-    EditCommands::Copy => 0x43,  // c
-    EditCommands::Cut => 0x58,   // x
-    EditCommands::Paste => 0x56, // v
+    EditCommand::Copy => 0x43,      // c
+    EditCommand::Cut => 0x58,       // x
+    EditCommand::Paste => 0x56,     // v
+    EditCommand::SelectAll => 0x41, // a
   };
 
   unsafe {
@@ -449,7 +460,7 @@ fn execute_edit_command(command: EditCommands) {
 // Convert a hotkey to an accelerator.
 fn convert_accelerator(id: u16, key: Accelerator) -> Option<winuser::ACCEL> {
   let mut virt_key = winuser::FVIRTKEY;
-  let key_mods: ModifiersState = key.mods.into();
+  let key_mods: ModifiersState = key.mods;
   if key_mods.control_key() {
     virt_key |= winuser::FCONTROL;
   }
@@ -486,7 +497,7 @@ fn convert_accelerator(id: u16, key: Accelerator) -> Option<winuser::ACCEL> {
 
 // Format the hotkey in a Windows-native way.
 fn format_hotkey(key: Accelerator, s: &mut String) {
-  let key_mods: ModifiersState = key.mods.into();
+  let key_mods: ModifiersState = key.mods;
   if key_mods.control_key() {
     s.push_str("Ctrl+");
   }
@@ -500,42 +511,42 @@ fn format_hotkey(key: Accelerator, s: &mut String) {
     s.push_str("Windows+");
   }
   match &key.key {
-    KeyCode::KeyA => s.push_str("A"),
-    KeyCode::KeyB => s.push_str("B"),
-    KeyCode::KeyC => s.push_str("C"),
-    KeyCode::KeyD => s.push_str("D"),
-    KeyCode::KeyE => s.push_str("E"),
-    KeyCode::KeyF => s.push_str("F"),
-    KeyCode::KeyG => s.push_str("G"),
-    KeyCode::KeyH => s.push_str("H"),
-    KeyCode::KeyI => s.push_str("I"),
-    KeyCode::KeyJ => s.push_str("J"),
-    KeyCode::KeyK => s.push_str("K"),
-    KeyCode::KeyL => s.push_str("L"),
-    KeyCode::KeyM => s.push_str("M"),
-    KeyCode::KeyN => s.push_str("N"),
-    KeyCode::KeyO => s.push_str("O"),
-    KeyCode::KeyP => s.push_str("P"),
-    KeyCode::KeyQ => s.push_str("Q"),
-    KeyCode::KeyR => s.push_str("R"),
-    KeyCode::KeyS => s.push_str("S"),
-    KeyCode::KeyT => s.push_str("T"),
-    KeyCode::KeyU => s.push_str("U"),
-    KeyCode::KeyV => s.push_str("V"),
-    KeyCode::KeyW => s.push_str("W"),
-    KeyCode::KeyX => s.push_str("X"),
-    KeyCode::KeyY => s.push_str("Y"),
-    KeyCode::KeyZ => s.push_str("Z"),
-    KeyCode::Digit0 => s.push_str("0"),
-    KeyCode::Digit1 => s.push_str("1"),
-    KeyCode::Digit2 => s.push_str("2"),
-    KeyCode::Digit3 => s.push_str("3"),
-    KeyCode::Digit4 => s.push_str("4"),
-    KeyCode::Digit5 => s.push_str("5"),
-    KeyCode::Digit6 => s.push_str("6"),
-    KeyCode::Digit7 => s.push_str("7"),
-    KeyCode::Digit8 => s.push_str("8"),
-    KeyCode::Digit9 => s.push_str("9"),
+    KeyCode::KeyA => s.push('A'),
+    KeyCode::KeyB => s.push('B'),
+    KeyCode::KeyC => s.push('C'),
+    KeyCode::KeyD => s.push('D'),
+    KeyCode::KeyE => s.push('E'),
+    KeyCode::KeyF => s.push('F'),
+    KeyCode::KeyG => s.push('G'),
+    KeyCode::KeyH => s.push('H'),
+    KeyCode::KeyI => s.push('I'),
+    KeyCode::KeyJ => s.push('J'),
+    KeyCode::KeyK => s.push('K'),
+    KeyCode::KeyL => s.push('L'),
+    KeyCode::KeyM => s.push('M'),
+    KeyCode::KeyN => s.push('N'),
+    KeyCode::KeyO => s.push('O'),
+    KeyCode::KeyP => s.push('P'),
+    KeyCode::KeyQ => s.push('Q'),
+    KeyCode::KeyR => s.push('R'),
+    KeyCode::KeyS => s.push('S'),
+    KeyCode::KeyT => s.push('T'),
+    KeyCode::KeyU => s.push('U'),
+    KeyCode::KeyV => s.push('V'),
+    KeyCode::KeyW => s.push('W'),
+    KeyCode::KeyX => s.push('X'),
+    KeyCode::KeyY => s.push('Y'),
+    KeyCode::KeyZ => s.push('Z'),
+    KeyCode::Digit0 => s.push('0'),
+    KeyCode::Digit1 => s.push('1'),
+    KeyCode::Digit2 => s.push('2'),
+    KeyCode::Digit3 => s.push('3'),
+    KeyCode::Digit4 => s.push('4'),
+    KeyCode::Digit5 => s.push('5'),
+    KeyCode::Digit6 => s.push('6'),
+    KeyCode::Digit7 => s.push('7'),
+    KeyCode::Digit8 => s.push('8'),
+    KeyCode::Digit9 => s.push('9'),
     KeyCode::Escape => s.push_str("Esc"),
     KeyCode::Delete => s.push_str("Del"),
     KeyCode::Insert => s.push_str("Ins"),
