@@ -16,7 +16,7 @@ pub use self::{
   keycode::{keycode_from_scancode, keycode_to_scancode},
   menu::{Menu, MenuItemAttributes},
   monitor::{MonitorHandle, VideoMode},
-  window::Window,
+  window::{hit_test, Window},
 };
 
 pub use self::icon::WinIcon as PlatformIcon;
@@ -32,6 +32,7 @@ mod system_tray;
 #[cfg(feature = "tray")]
 pub use self::system_tray::{SystemTray, SystemTrayBuilder};
 
+#[non_exhaustive]
 #[derive(Clone)]
 pub enum Parent {
   None,
@@ -44,6 +45,7 @@ pub struct PlatformSpecificWindowBuilderAttributes {
   pub parent: Parent,
   pub menu: Option<HMENU>,
   pub taskbar_icon: Option<Icon>,
+  pub skip_taskbar: bool,
   pub no_redirection_bitmap: bool,
   pub drag_and_drop: bool,
   pub preferred_theme: Option<Theme>,
@@ -58,6 +60,7 @@ impl Default for PlatformSpecificWindowBuilderAttributes {
       no_redirection_bitmap: false,
       drag_and_drop: true,
       preferred_theme: None,
+      skip_taskbar: false,
     }
   }
 }
@@ -90,6 +93,7 @@ impl DeviceId {
   }
 }
 
+#[non_exhaustive]
 #[derive(Debug)]
 pub enum OsError {
   CreationError(&'static str),
