@@ -56,6 +56,7 @@ use crate::{
   },
   window::{
     CursorIcon, Fullscreen, Theme, UserAttentionType, WindowAttributes, WindowId as RootWindowId,
+    BORDERLESS_RESIZE_INSET,
   },
 };
 
@@ -142,10 +143,7 @@ impl Window {
   }
 
   pub fn set_title(&self, text: &str) {
-    let text = OsStr::new(text)
-      .encode_wide()
-      .chain(Some(0).into_iter())
-      .collect::<Vec<_>>();
+    let text = util::to_wstring(text);
     unsafe {
       winuser::SetWindowTextW(self.window.0, text.as_ptr() as LPCWSTR);
     }
@@ -156,13 +154,29 @@ impl Window {
 
   #[inline]
   pub fn set_visible(&self, visible: bool) {
-    let window = self.window.clone();
-    let window_state = Arc::clone(&self.window_state);
-    self.thread_executor.execute_in_thread(move || {
-      WindowState::set_window_flags(window_state.lock(), window.0, |f| {
-        f.set(WindowFlags::VISIBLE, visible)
+    let prev = self.is_visible();
+    let skip_taskbar = self.window_state.lock().skip_taskbar;
+    // Hidden window also skips taskbar, we need to check if it conflicts with skip_taskbar state
+    // If it's moving from visible to hidden, we need to unset skip_taskbar
+    if prev && !visible && skip_taskbar {
+      self.set_skip_taskbar(false, false);
+    }
+
+    // If it's still the same, there's no need to set it again
+    if prev != visible {
+      let window = self.window.clone();
+      let window_state = Arc::clone(&self.window_state);
+      self.thread_executor.execute_in_thread(move || {
+        WindowState::set_window_flags(window_state.lock(), window.0, |f| {
+          f.set(WindowFlags::VISIBLE, visible)
+        });
       });
-    });
+    }
+
+    // If it's moving from hidden to visible, we set skip_taskbar back
+    if !prev && visible && skip_taskbar {
+      self.set_skip_taskbar(true, false);
+    }
   }
 
   #[inline]
@@ -492,7 +506,7 @@ impl Window {
           // string, so add it
           display_name.push(0);
 
-          let mut native_video_mode = video_mode.video_mode.native_video_mode.clone();
+          let mut native_video_mode = video_mode.video_mode.native_video_mode;
 
           let res = unsafe {
             winuser::ChangeDisplaySettingsExW(
@@ -738,7 +752,7 @@ impl Window {
 
   #[inline]
   pub fn is_menu_visible(&self) -> bool {
-    unsafe { winuser::GetMenu(self.hwnd()) != ptr::null_mut() }
+    unsafe { !winuser::GetMenu(self.hwnd()).is_null() }
   }
 
   #[inline]
@@ -762,22 +776,30 @@ impl Window {
   }
 
   #[inline]
-  pub(crate) fn set_skip_taskbar(&self, skip: bool) {
-    unsafe {
-      let mut taskbar_list: *mut ITaskbarList = std::mem::zeroed();
-      CoCreateInstance(
-        &CLSID_TaskbarList,
-        std::ptr::null_mut(),
-        CLSCTX_SERVER,
-        &ITaskbarList::uuidof(),
-        &mut taskbar_list as *mut _ as *mut _,
-      );
-      if skip {
-        (*taskbar_list).DeleteTab(self.hwnd() as _);
-      } else {
-        (*taskbar_list).AddTab(self.hwnd() as _);
+  pub(crate) fn set_skip_taskbar(&self, skip: bool, state: bool) {
+    // Update self skip_taskbar state if true
+    if state {
+      let mut window_state = self.window_state.lock();
+      window_state.skip_taskbar = skip;
+    }
+
+    if self.is_visible() {
+      unsafe {
+        let mut taskbar_list: *mut ITaskbarList = std::mem::zeroed();
+        CoCreateInstance(
+          &CLSID_TaskbarList,
+          std::ptr::null_mut(),
+          CLSCTX_SERVER,
+          &ITaskbarList::uuidof(),
+          &mut taskbar_list as *mut _ as *mut _,
+        );
+        if skip {
+          (*taskbar_list).DeleteTab(self.hwnd() as _);
+        } else {
+          (*taskbar_list).AddTab(self.hwnd() as _);
+        }
+        (*taskbar_list).Release();
       }
-      (*taskbar_list).Release();
     }
   }
 }
@@ -810,10 +832,7 @@ unsafe fn init<T: 'static>(
   pl_attribs: PlatformSpecificWindowBuilderAttributes,
   event_loop: &EventLoopWindowTarget<T>,
 ) -> Result<Window, RootOsError> {
-  let title = OsStr::new(&attributes.title)
-    .encode_wide()
-    .chain(Some(0).into_iter())
-    .collect::<Vec<_>>();
+  let title = util::to_wstring(&attributes.title);
 
   // registering the window class
   let class_name = register_window_class(&attributes.window_icon, &pl_attribs.taskbar_icon);
@@ -911,6 +930,7 @@ unsafe fn init<T: 'static>(
       scale_factor,
       current_theme,
       pl_attribs.preferred_theme,
+      pl_attribs.skip_taskbar,
     );
     let window_state = Arc::new(Mutex::new(window_state));
     WindowState::set_window_flags(window_state.lock(), real_window.0, |f| *f = window_flags);
@@ -924,6 +944,8 @@ unsafe fn init<T: 'static>(
     menu: None,
   };
 
+  win.set_skip_taskbar(pl_attribs.skip_taskbar, false);
+
   let dimensions = attributes
     .inner_size
     .unwrap_or_else(|| PhysicalSize::new(800, 600).into());
@@ -935,7 +957,7 @@ unsafe fn init<T: 'static>(
   }
   win.set_visible(attributes.visible);
 
-  if let Some(_) = attributes.fullscreen {
+  if attributes.fullscreen.is_some() {
     win.set_fullscreen(attributes.fullscreen);
     force_window_active(win.window.0);
   }
@@ -947,7 +969,7 @@ unsafe fn init<T: 'static>(
   if let Some(window_menu) = attributes.window_menu {
     let event_loop_runner = event_loop.runner_shared.clone();
     let window_handle = win.raw_window_handle();
-    let window_id = RootWindowId(win.id().clone());
+    let window_id = RootWindowId(win.id());
     let menu_handler = menu::MenuHandler::new(
       Box::new(move |event| {
         if let Ok(e) = event.map_nonuser_event() {
@@ -958,11 +980,9 @@ unsafe fn init<T: 'static>(
       Some(window_id),
     );
 
-    win.menu = menu::initialize(window_menu, window_handle, menu_handler).map(|m| HMenuWrapper(m));
+    win.menu = menu::initialize(window_menu, window_handle, menu_handler).map(HMenuWrapper);
   }
 
-  win.set_skip_taskbar(pl_attribs.skip_taskbar);
-
   Ok(win)
 }
 
@@ -970,10 +990,7 @@ unsafe fn register_window_class(
   window_icon: &Option<Icon>,
   taskbar_icon: &Option<Icon>,
 ) -> Vec<u16> {
-  let class_name: Vec<_> = OsStr::new("Window Class")
-    .encode_wide()
-    .chain(Some(0).into_iter())
-    .collect();
+  let class_name = util::to_wstring("Window Class");
 
   let h_icon = taskbar_icon
     .as_ref()
@@ -1018,9 +1035,9 @@ unsafe extern "system" fn window_proc(
 
   match msg {
     winuser::WM_NCCALCSIZE => {
-      // Check if userdata is set and if the value of it is true
+      // Check if userdata is set and if the value of it is true (window wants to be borderless)
       if userdata != 0 && *(userdata as *mut bool) == true {
-        // adjust the maximized borderless window to fill the work area rectangle of the display monitor
+        // adjust the maximized borderless window so it doesn't cover the taskbar
         if util::is_maximized(window) {
           let monitor = monitor::current_monitor(window);
           if let Ok(monitor_info) = monitor::get_monitor_info(monitor.hmonitor()) {
@@ -1028,7 +1045,7 @@ unsafe extern "system" fn window_proc(
             params.rgrc[0] = monitor_info.rcWork;
           }
         }
-        0 // must return a value here, otherwise the window won't be borderless (without decorations)
+        0 // return 0 here to make the windowo borderless
       } else {
         winuser::DefWindowProcW(window, msg, wparam, lparam)
       }
@@ -1082,7 +1099,7 @@ unsafe fn taskbar_mark_fullscreen(handle: HWND, fullscreen: bool) {
   TASKBAR_LIST.with(|task_bar_list_ptr| {
     let mut task_bar_list = task_bar_list_ptr.get();
 
-    if task_bar_list == ptr::null_mut() {
+    if task_bar_list.is_null() {
       use winapi::shared::winerror::S_OK;
 
       let hr = combaseapi::CoCreateInstance(
@@ -1144,7 +1161,7 @@ pub fn hit_test(hwnd: HWND, cx: i32, cy: i32) -> LRESULT {
     let mut window_rect: RECT = mem::zeroed();
     if GetWindowRect(hwnd, <*mut _>::cast(&mut window_rect)) == TRUE {
       const CLIENT: i32 = 0b0000;
-      const LEFT: i32 = 00001;
+      const LEFT: i32 = 0b0001;
       const RIGHT: i32 = 0b0010;
       const TOP: i32 = 0b0100;
       const BOTTOM: i32 = 0b1000;
@@ -1153,8 +1170,6 @@ pub fn hit_test(hwnd: HWND, cx: i32, cy: i32) -> LRESULT {
       const BOTTOMLEFT: i32 = BOTTOM | LEFT;
       const BOTTOMRIGHT: i32 = BOTTOM | RIGHT;
 
-      let fake_border = 3; // change this to manipulate how far inside the window, the resize can happen
-
       let RECT {
         left,
         right,
@@ -1162,12 +1177,13 @@ pub fn hit_test(hwnd: HWND, cx: i32, cy: i32) -> LRESULT {
         top,
       } = window_rect;
 
-      let hit_result = LEFT * (if cx < (left + fake_border) { 1 } else { 0 })
-        | RIGHT * (if cx >= (right - fake_border) { 1 } else { 0 })
-        | TOP * (if cy < (top + fake_border) { 1 } else { 0 })
-        | BOTTOM * (if cy >= (bottom - fake_border) { 1 } else { 0 });
+      #[rustfmt::skip]
+      let result = (LEFT * (if cx < (left + BORDERLESS_RESIZE_INSET) { 1 } else { 0 }))
+        | (RIGHT * (if cx >= (right - BORDERLESS_RESIZE_INSET) { 1 } else { 0 }))
+        | (TOP * (if cy < (top + BORDERLESS_RESIZE_INSET) { 1 } else { 0 }))
+        | (BOTTOM * (if cy >= (bottom - BORDERLESS_RESIZE_INSET) { 1 } else { 0 }));
 
-      match hit_result {
+      match result {
         CLIENT => HTCLIENT,
         LEFT => HTLEFT,
         RIGHT => HTRIGHT,
